==> plot_window-0.0.21.tar.gz: lib/point_smoother.dart <==
part of plot_window;

typedef num Reducer(List<num> nums);

class AverageMethod {
  String name;
  Reducer reduce;

  static Map<String, AverageMethod> CACHE = new Map<String, AverageMethod>();
  
  static AverageMethod ARITHMETIC = new AverageMethod._intern("Arithmetic",
      (List<num> nums)=>nums.fold(0, (num prev, num elem)=>prev+elem) / nums.length);
    
  static AverageMethod GEOMETRIC = new AverageMethod._intern("Geometric",
      (List<num> nums)=>math.exp(nums.fold(0, (num prev, num elem)=>prev + math.log(elem))/nums.length));
  bool operator ==(Object other) {
    if(!other is AverageMethod) return false;
    AverageMethod otherAverageMethod = other;
    return this.name==otherAverageMethod.name;
  }

  AverageMethod._intern(this.name, this.reduce) {
    CACHE[this.name] = this;
  }
  
  static init() {
    ARITHMETIC; GEOMETRIC;
  }

}


class PointSmoother {
  AverageMethod average;
  int width;

  String toString()=>"${this.average.name} (${this.width})";
  
  bool operator ==(Object other) {
    if(!other is PointSmoother) return false;
    PointSmoother otherPointSmoother = other;
    return this.average==otherPointSmoother.average && this.width==otherPointSmoother.width;
  }

  PointSmoother(this.average, this.width);

  List<Point>smooth(List<Point> points) {
    List<Point> ret = new List<Point>();
    for(int i=0; i<points.length - width; i++) {
      var sublist = points.skip(i).take(width).toList();
      ret.add(new Point(sublist[width~/2].x, average.reduce(sublist.map((Point p)=>p.y).toList())));
    }
    return ret;
  }
  
}

==> plot_window-0.0.22.tar.gz: lib/point_smoother.dart <==
part of plot_window;

typedef num NumberFunction(num n);

class AverageMethod {
  String name;
  NumberFunction before;
  NumberFunction after;

  static Map<String, AverageMethod> CACHE = new Map<String, AverageMethod>();
  
  static AverageMethod ARITHMETIC = new AverageMethod._intern("Arithmetic",
      (num n)=>n, (num n)=>n);
    
  static AverageMethod GEOMETRIC = new AverageMethod._intern("Geometric",
      (num n)=>math.log(n), (num n)=>math.exp(n));

  bool operator ==(Object other) {
    if(!other is AverageMethod) return false;
    AverageMethod otherAverageMethod = other;
    return this.name==otherAverageMethod.name;
  }

  AverageMethod._intern(this.name, this.reduce) {
    CACHE[this.name] = this;
  }
  
  static init() {
    ARITHMETIC; GEOMETRIC;
  }

}


class PointSmoother {
  AverageMethod average;
  int width;

  String toString()=>"${this.average.name} (${this.width})";
  
  bool operator ==(Object other) {
    if(!other is PointSmoother) return false;
    PointSmoother otherPointSmoother = other;
    return this.average==otherPointSmoother.average && this.width==otherPointSmoother.width;
  }

  PointSmoother(this.average, this.width);

  List<Point>smooth(List<Point> points) {
    if (points.length < 2 * width + 1) { throw "points list too short"; }

    var input = points.map((Point p)=>average.before(p.y)/(2*width+1));
    var vals = new List<num>(points.length - 2*width -1);
    vals[0] = input.sublist(0, 2*width +1).fold(0.0, (num total, num elem )=>total+elem);
    for (int i=1; i<vals.length; i++) {
      vals[i] = vals[i-1] - input[i-1] + input[i+2*width];
    }

    for (int i=0; i<vals.length; i++) {
      ret.add(new Point(points[width+i].x, average.after(vals[i])));
    }
    return ret;
  }
  
}
